	OPTION EXPLICIT
	OPTION DEFAULT NONE
	CPU 48
	GOTO TestSuite


'************************************************************************************* 
' 
' Peter Mather
' 3.0 		2015-06-25
' 3.0.1 	2015-06-26 	Modified by Peter Carnegie, 
'						Changed S1963_ to S1963. 
'						Added S1963.PinDefs call - eliminates need for S1963 global constants
'						Changed T. to XPT.
'						Changed T_ to XPT.
'						Changed all Global variables in Touch to prefix XPT.
'						Changed XPT.Init to take IRQ and CS as arguments - eliminates XPT global constants
  
 
' Graphics library with drivers for SSD1963 display
' 9 bit parallel output from Micromite MX170 - flexible pinout

' Touch Input Library with drivers for XPT2046

' drawing primitives assume a graphics display where top left is point 0, 0
' function calls are as compatible as possible with the in-built graphics routines as documented for version 4.7
	
' Define which pins are used to interface with the display
' Call S1963.PinDefs BEFORE calling anything else in S1963
' Sub S1963.PinDefs(D0,D1,D2,D3,D4,D5,D6,D7,D8,D9,RS,RD,WR,RST)
'	  D0-D9 Pins used for DB0-DB9 see notes below about optimisation
'	  RS Pin used for RS
'     RD Pin used for RD
'     WR Pin used for WR
'     RST Pin used to generate Reset signal to display controller
'	  For optimised update and scroll functionality, otherwise any valid pin
' 	  D0, Use 25
'     D1, Use 26
'     D2, Use 27
'     D3, Use 36
'     D4, Use 37
'     D5, Use 38
'     D6, Use 2
' 	  D7, Use 3
'     D8, Use 4
' 	  D9, Use 5
'	 
'
'initialise display:
'SUB S1963.init(size)
'     display size in inches, can be 4.3, 5, or 7
'
'clear screen:
'SUB S1963.CLS(colour)
'    clear the screen to the colour specified
'    if the colour is omitted the screen is cleared to black
'
'set screen orientation:
'SUB S1963.ROTATE(orientation)
'    valid values are PORTRAIT:0, RPORTRAIT:1, LANDSCAPE:2, RLANDSCAPE:3
'
'plot PIXEL:
'SUB S1963.PIXEL(x, y, c)
'    x, y coordinates of PIXEL, colour
'
'draw circle:
'SUB S1963.CIRCLE(x, y, r, c, f)
'    x, y coordinates of centre, radius, colour of outline, colour to fill
'    if colour to fill is omitted an outline only is drawn
'    to fill with black use the special function S1963.BLACK() as the colour
'    to generate a circle in a single solid colour use the same colour for the outline and the fill
'
'draw triangle:
'SUB S1963.TRIANGLE(x0, y0, x1, y1, 	x2, y2, c, f)
'    three coordinate pairs, colour of outline, colour to fill
'    if colour to fill is omitted an outline only is drawn
'    to fill with black use the special function S1963.BLACK() as the colour
'    to generate a triangle in a single solid colour use the same colour for the outline and the fill
'
'print text:
'SUB S1963.TEXT(x, y, text$, font, size, forecolour, backcolour)
'    x, y coordinates of top left of first character, font number, text string, size, c,  foreground colour,  background colour
'    if the background colour is omitted black will be used
'
'print bitmap:
'SUB S1963.BITMAP(x, y, bitmap, width, height, scale, forecolour, backcolour)
'    x, y coordinates of top left of the bitmap, bitmap, size, character orientation,  foreground colour,  background colour
'    the bitmap must be specifed as an array of integers
'    to output bitmap contained within a single integer use array size 1 and allocate the bimap to element 0
'
'draw line:
'SUB S1963.LINE(x0, y0, x1, y1, colour)
'    x, y of start line, x, y of end line, foreground colour,  background colour
'
'draw rectangle:
'SUB S1963.BOX(x, y, w, h, c, f)
'    x, y coordinate of top left, width, height, colour of outline, colour to fill
'    if colour to fill is omitted an outline only is drawn
'    to fill with black use the special function S1963.BLACK() as the colour
'    to generate a rectangle in a single solid colour use the same colour for the outline and the fill
'
'draw rounded rectangle:
'SUB S1963.RBOX(x, y, w, h, r, c, f)
'    x, y coordinates of top left, width, height, radius of corner, colour of outline, colour to fill
'    if colour to fill is omitted an outline only is drawn
'    to fill with black use the special function S1963.BLACK() as the colour
'    to generate a rounded rectangle in a single solid colour use the same colour for the outline and the fill
'
'scroll region:
'SUB S1963.SCROLL(x ,y, w, h)
'    x, y coordinate of top left, width, height of scroll area
'
' screen horizontal resolution in default orientation:  S1963.MM.HRES.DEFAULT.DEFAULT
' screen vertical resolution in default orientation:  S1963.MM.VRES.DEFAULT.DEFAULT
' screen horizontal resolution in current orientation:  S1963.MM.HRES.DEFAULT
' screen vertical resolution in current orientation:  S1963.MM.VRES.DEFAULT
'
'
' Touch input routines
'
'initialise touch:
'SUB XPT.init(IRQ,CSxl%, xr%, yt%, yb%, xs%, ys%, n%)
'     calibration parameters for the touch screen, if not specified will automatically call XPT.CALIBRATE
'     number of samples at each touch to minimise noise, defaults to 5
'     CS: chip select pin for XPT touch controller, any valid pin
'     IRQ: interrupt pin for XPT touch controller, any valid pin
'     S1963_default_rotation: default screen orientation for this display type, in the case of the S1963 this is LANDSCAPE:2
'
'rotate screen keeping touch input correct :
'SUB XPT.ROTATE(N)
'    valid values for N are PORTRAIT:0, RPORTRAIT:1, LANDSCAPE:2, RLANDSCAPE:3
'    note if S1963.ROTATE is used rather than XPT.ROTATE when touch is active subsequent touch coordinates will be incorrect
'
'calibrate touch:
'SUB XPT.CALIBRATE
'    no parameters
'    sets and prints the following values
'    XPT.XSAMPLE0, XPT.YSAMPLE0, XPT.XLEFT0, XPT.XRIGHT0, XPT.YTOP0, XPT.YBOTTOM0
'    for any given display these will be fixed so can be set as constants for future use
'    this allows the calibrate routine to be removed for working systems
'    the related variables XPT.XSAMPLE, XPT.YSAMPLE, XPT.XLEFT, XPT.XRIGHT, XPT.YTOP, XPT.YBOTTOM are set for the default orientation
'    these are changed by XPT.ROTATE from the values stored in XPT.XSAMPLE0 etc.
'
'get touch coordinates:
'FUNCTION XPT.SCREENPOS(x, y, t)
'    returns true if a touch has occured within the timeout period
'    returns false if the timeout has occured
'    if true, x and y coordinates of position touched
'    timeout after which a false return will occur if no touch
'
'

' Display specific routines
' choose the size of display 4.3 or 5 or 7
SUB S1963.init(S1963.displaysize AS FLOAT) 

	'Global variables
	'Command codes for S1963 CFunction
	DIM INTEGER S1963.pix=0, S1963.ffr=1, S1963.filltri=2, S1963.drawline=3, S1963.wc=4, S1963.wd=5
	DIM INTEGER S1963.fpstring=7, S1963.fcirch=8, S1963.circh=9, S1963.Bits=10, S1963.scrolled=11, S1963.initialise=12

	'Orientation codes
	DIM INTEGER LANDSCAPE=2, RLANDSCAPE=3, PORTRAIT=0, RPORTRAIT=1
	
	DIM INTEGER S1963.MM.HRES.DEFAULT, S1963.MM.VRES.DEFAULT, S1963.MM.HRES, S1963.MM.VRES
	DIM INTEGER S1963.PAG_COL_ADDR = &H20
	DIM INTEGER S1963.COL_ADDR     = &H40
	DIM INTEGER S1963.PAGE_ADDR    = &H80

	IF S1963.displaysize<4.5 THEN
		S1963.MM.HRES.DEFAULT=480: S1963.MM.VRES.DEFAULT=272
	ELSE
		S1963.MM.HRES.DEFAULT=800: S1963.MM.VRES.DEFAULT=480
	ENDIF
	S1963.VAR(S1963.OPTIMISE)=0
	IF S1963.VAR(S1963.DB0_I)=25 AND S1963.VAR(S1963.DB1_I)=26 AND S1963.VAR(S1963.DB2_I)=27 AND S1963.VAR(S1963.DB3_I)=36 THEN
		IF S1963.VAR(S1963.DB4_I)=37 AND S1963.VAR(S1963.DB5_I)=38 AND S1963.VAR(S1963.DB6_I)=2 AND S1963.VAR(S1963.DB7_I)=3 THEN
			IF S1963.VAR(S1963.DB8_I)=4 AND S1963.VAR(S1963.DB9_I)=5 THEN
				S1963.VAR(S1963.OPTIMISE)=1
			ENDIF
		ENDIF
	ENDIF
	
	S1963.VAR(S1963.wi)=S1963.MM.HRES.DEFAULT :  S1963.VAR(S1963.ht)=S1963.MM.VRES.DEFAULT
	S1963.VAR(S1963.PORTS)=S1963(S1963.initialise,S1963.VAR())
	
	'Reset the display controller
	PIN(S1963.VAR(S1963.RST_I))=1:  SETPIN S1963.VAR(S1963.RST_I),DOUT
	PAUSE 100
	PIN(S1963.VAR(S1963.RST_I))=0
	PAUSE 100
	PIN(S1963.VAR(S1963.RST_I))=1
	PAUSE 100
	
	IF S1963.displaysize<4.5 THEN
		S1963.CFG(&HE2,3,&H23,&H02,&H54)	   
	ELSE
		S1963.CFG(&HE2,3,&H1E,&H02,&H54)	   
	ENDIF
	
	S1963.CFG(&HE0,1,&H01): PAUSE 10 		' PLL enable
	S1963.CFG(&HE0,1,&H03):	PAUSE 10
	S1963.CFG(&H01,0): PAUSE 100		' software reset
	
	IF S1963.displaysize<4.5 THEN
		S1963.CFG(&HE6,3,&H01,&H1F,&HFF)
		S1963.CFG(&HB0,7,&H20,&H00,&H01,&HDF,&H01,&H0F,&H00)		'LCD SPECIFICATION
		S1963.CFG(&HB4,8,&H02,&H13,&H00,&H08,&H2B,&H00,&H02,&H00)	
		S1963.CFG(&HB6,7,&H01,&H20,&H00,&H04,&H0C,&H00,&H02)	
	ELSE
		S1963.CFG(&HE6,3,&H03,&HFF,&HFF)
		S1963.CFG(&HB0,7,&H24,&H00,&H03,&H1F,&H01,&HDF,&H00)		'LCD SPECIFICATION
		S1963.CFG(&HB4,8,&H03,&HA0,&H00,&H2E,&H30,&H00,&H0F,&H00)
		S1963.CFG(&HB6,7,&H02,&H0D,&H00,&H10,&H10,&H00,&H08)	
	ENDIF
	
	S1963.CFG(&HBA,1,&H0F)
	S1963.CFG(&HB8,2,&H07,&H01)
	S1963.CFG(&H36,1,&H00)		'rotation
	S1963.CFG(&HF0,1,&H06)		'PIXEL data interface
	S1963.CFG(&H3A,1,&B01100000)		'PIXEL data interface
	S1963.CFG(&H26,1,&H01) : PAUSE 1		'gamma curve
	S1963.CFG(&H29,0)		'display on
	S1963.CFG(&HBE,6,&H06,&HF0,&H01,&HF0,&H00,&H00)		'set PWM for B/L
	S1963.CFG(&Hd0,1,&H0D) 
	S1963.rotate(landscape)
END SUB


' Set up the pin definitions
' Call S1963.PinDefs BEFORE calling anything else in S1963 
' RS, any valid pin
' RD, any valid pin, set to zero if tied high, scroll functionality not available if zero
' WR, any valid pin
' D0, Use 25 for optimised update and scroll functionality, otherwise any valid pin
' D1, Use 26 for optimised update and scroll functionality, otherwise any valid pin
' D2, Use 27 for optimised update and scroll functionality, otherwise any valid pin
' D3, Use 36 for optimised update and scroll functionality, otherwise any valid pin
' D4, Use 37 for optimised update and scroll functionality, otherwise any valid pin
' D5, Use 38 for optimised update and scroll functionality, otherwise any valid pin
' D6, Use 2 for optimised update and scroll functionality, otherwise any valid pin
' D7, Use 3 for optimised update and scroll functionality, otherwise any valid pin
' D8, Use 4 for optimised update and scroll functionality, otherwise any valid pin
' D9, Use 5 for optimised update and scroll functionality, set to zero if not using PortC pins for databus
' RST, any valid pin
'
SUB S1963.PinDefs(D0%,D1%,D2%,D3%,D4%,D5%,D6%,D7%,D8%,D9%,RS%,RD%,WR%,RST%)

	'Holds Parameters for use in the Cfunction
	DIM INTEGER S1963.VAR(22) 
	
	'Subscripts in S1963.VAR for each parameter
	DIM INTEGER S1963.Default_Rotation=2 'default screen orientation for this display type, in this case LANDSCAPE
	DIM INTEGER S1963.wi=0,S1963.ht=1,S1963.rot=2,S1963.nr=3,S1963.X=4,S1963.Y=5,S1963.seed=6  
	DIM INTEGER S1963.RS_I=7, S1963.WR_I=8
	
	'Subscripts for the 10 bit data bus - why don't the DB pins start at subscript 0 ?
	DIM INTEGER S1963.DB0_I=9, S1963.DB1_I=10, S1963.DB2_I=11, S1963.DB3_I=12,S1963.DB4_I=13
	DIM INTEGER S1963.DB5_I=14, S1963.DB6_I=15, S1963.DB7_I=16, S1963.DB8_I=17
	DIM INTEGER S1963.RD_I=18, S1963.PORTS=19, S1963.OPTIMISE=20, S1963.RST_I=21, S1963.DB9_I=22
	pin(RST%)=1:  SETPIN RST%,dout	
	'the CFunction will execute the SETPIN commands itself
	S1963.VAR(S1963.RST_I)=RST% : 
	S1963.VAR(S1963.RS_I)=RS%   :  S1963.VAR(S1963.WR_I)=WR%  :  S1963.VAR(S1963.RD_I)=RD%
	S1963.VAR(S1963.DB0_I)=D0%  :  S1963.VAR(S1963.DB1_I)=D1% :  S1963.VAR(S1963.DB2_I)=D2%
	S1963.VAR(S1963.DB3_I)=D3%  :  S1963.VAR(S1963.DB4_I)=D4% :  S1963.VAR(S1963.DB5_I)=D5%
	S1963.VAR(S1963.DB6_I)=D6%  :  S1963.VAR(S1963.DB7_I)=D7% :  S1963.VAR(S1963.DB8_I)=D8%
	S1963.VAR(S1963.DB9_I)=D9%
	
END SUB

SUB S1963.CFG(c%, n AS INTEGER, d1%, d2%, d3%, d4%, d5%, d6%, d7%, d8%)
LOCAL INTEGER i
   i=S1963(S1963.wc, c%, S1963.VAR())
   IF n>0 THEN i=S1963(S1963.wd, d1%, S1963.VAR())
   IF n>1 THEN i=S1963(S1963.wd, d2%, S1963.VAR())
   IF n>2 THEN i=S1963(S1963.wd, d3%, S1963.VAR())
   IF n>3 THEN i=S1963(S1963.wd, d4%, S1963.VAR())
   IF n>4 THEN i=S1963(S1963.wd, d5%, S1963.VAR())
   IF n>5 THEN i=S1963(S1963.wd, d6%, S1963.VAR())
   IF n>6 THEN i=S1963(S1963.wd, d7%, S1963.VAR())
   IF n>7 THEN i=S1963(S1963.wd, d8%, S1963.VAR())
END SUB
'
' Generic drawing routines
'
'
SUB S1963.CLS(c%)
  LOCAL INTEGER j,i=S1963.VAR(S1963.rot)
  S1963.VAR(S1963.rot)=LANDSCAPE
  j=S1963(S1963.ffr, c%, 0,0,S1963.VAR(S1963.wi),S1963.VAR(S1963.ht), S1963.VAR())
  S1963.VAR(S1963.rot)=i
END SUB
'
SUB S1963.CIRCLE(x%, y%, r%, c%, f%)
  LOCAL i%
  IF(f%) THEN 
    i%=S1963(S1963.ffr, f%, x%, y%-r%, 1, 2*r%+1,  S1963.VAR())
    i%=S1963(S1963.fcircH, x%, y%, r%, 3, 0, f%, S1963.VAR())
  ENDIF
  IF(f%<>c%) THEN i%=S1963(S1963.circh, c%, x%, y%, r%, 31, S1963.VAR())
END SUB
 '
FUNCTION S1963.black()AS INTEGER
   S1963.black=&H8000000000000000
END FUNCTION
'
SUB S1963.PIXEL(x AS INTEGER, y AS INTEGER,  c AS INTEGER) 
  LOCAL i%=S1963(S1963.pix, c, x, y, S1963.VAR())
END SUB
 '
SUB S1963.TRIANGLE(x0 AS INTEGER, y0 AS INTEGER, x1 AS INTEGER, y1 AS INTEGER, x2 AS INTEGER, y2 AS INTEGER, c AS INTEGER, f AS INTEGER)
  LOCAL i%
  IF(f) THEN i%=S1963(S1963.filltri, x0,y0,x1,y1,x2,y2,c,S1963.VAR())
  IF(f<>c) THEN
     S1963.LINE(x0, y0, x1, y1, c)
     S1963.LINE(x1, y1, x2, y2, c)
     S1963.LINE(x2, y2, x0, y0, c)
  ENDIF
END SUB
'
SUB S1963.TEXT(x%, y%, text$, font%, sz%, c%, f%) 
  LOCAL j%=font% AND 1
  LOCAL INTEGER i=S1963(S1963.fpstring,j%,x%,y%,sz%,c%,f%,S1963.VAR(),text$)
END SUB
'
SUB S1963.LINE(x0%, y0%, x1%, y1%, c%)
   LOCAL i%=S1963(S1963.drawline, c%, x0%, y0%, x1%, y1%,  S1963.VAR())
END SUB
'
SUB S1963.BOX(x AS INTEGER, y AS INTEGER, w AS INTEGER, h AS INTEGER, c AS INTEGER, f AS INTEGER)
  LOCAL INTEGER a, i
  IF(f) THEN i=S1963(S1963.ffr, f, x, y, w, h,  S1963.VAR()) 
  IF(f<>c) THEN
     i=S1963(S1963.ffr, c, x, y, w, 1,  S1963.VAR()) 
     a=y+h-1
     i=S1963(S1963.ffr, c, x, a, w, 1,  S1963.VAR()) 
     i=S1963(S1963.ffr, c, x, y, 1, h,  S1963.VAR()) 
     a=x+w-1
     i=S1963(S1963.ffr, c, a, y, 1, h,  S1963.VAR()) 
  ENDIF
END SUB
'
SUB S1963.BITMAP(x%, y%, array%(), w%, h%, sz%, c%, f%)
  LOCAL i%=S1963(S1963.Bits, x%,y%,w%,h%,sz%,c%,f%,array%(),S1963.VAR())  
END SUB
'
SUB S1963.RBOX(x AS INTEGER, y AS INTEGER, w AS INTEGER, h AS INTEGER, r AS INTEGER, c AS INTEGER, f AS INTEGER)
  LOCAL INTEGER i
  IF(f) THEN
	  i=S1963(S1963.ffr, f, x+r, y, w-2*r, h,  S1963.VAR()) 
	  i=S1963(S1963.fcircH,x+w-r-1, y+r, r, 1, h-2*r-1, f, S1963.VAR())
	  i=S1963(S1963.fcircH,x+r, y+r, r, 2, h-2*r-1, f, S1963.VAR())
  ENDIF
  IF(f<>c) THEN
	  i=S1963(S1963.ffr, c, x+r, y, w-2*r, 1,  S1963.VAR()) 
	  i=S1963(S1963.ffr, c, x+r, y+h-1, w-2*r, 1,  S1963.VAR()) 
	  i=S1963(S1963.ffr, c, x, y+r, 1, h-2*r,  S1963.VAR()) 
	  i=S1963(S1963.ffr, c, x+w-1, y+r, 1, h-2*r,  S1963.VAR()) 
	  i=S1963(S1963.circh, c, x+r, y+r, r, 1,S1963.VAR())
	  i=S1963(S1963.circh,  c, x+w-r-1, y+r, r, 2,S1963.VAR())
	  i=S1963(S1963.circh,  c, x+w-r-1, y+h-r-1, r, 4,S1963.VAR())
	  i=S1963(S1963.circh,  c, x+r, y+h-r-1, r, 8,S1963.VAR())
  ENDIF
END SUB
'
SUB S1963.SCROLL(x%, y%, w%, h%)
   IF (S1963.VAR(S1963.OPTIMISE)) AND  S1963.VAR(S1963.RD_I)) THEN
      LOCAL i%=S1963(S1963.SCROLLed,x%,y%,w%,h%,0,XPT.rbuf(),S1963.VAR())
   ENDIF
END SUB
'
SUB S1963.ROTATE(x%) 'sets screen rotation: 0=PORTRAIT, 1=RPORTRAIT, 2=LANDSCAPE, 3-RLANDSCAPE
    S1963.VAR(S1963.rot)=(x% AND 3)
    LOCAL o% = 0
    SELECT CASE x%
        CASE PORTRAIT
		       S1963.MM.VRES=S1963.MM.HRES.DEFAULT
		       S1963.MM.HRES=S1963.MM.VRES.DEFAULT
           o% = S1963.PAGE_ADDR OR S1963.PAG_COL_ADDR
        CASE RPORTRAIT
		       S1963.MM.VRES=S1963.MM.HRES.DEFAULT
		       S1963.MM.HRES=S1963.MM.VRES.DEFAULT
           o% = S1963.COL_ADDR OR S1963.PAG_COL_ADDR
        CASE LANDSCAPE
		       S1963.MM.VRES=S1963.MM.VRES.DEFAULT
		       S1963.MM.HRES=S1963.MM.HRES.DEFAULT
        CASE RLANDSCAPE
		       S1963.MM.VRES=S1963.MM.VRES.DEFAULT
		       S1963.MM.HRES=S1963.MM.HRES.DEFAULT
           o% = S1963.PAGE_ADDR OR  S1963.COL_ADDR
    END SELECT
    S1963.CFG(&H36,1,o%) ' set address mode
END SUB
'
' 
' Touch specific routines
'
SUB XPT.init(IRQ%, CS%, xl%, xr%, yt%, yb%, xs%, ys%, n%)
  DIM INTEGER XPT.NSAMPLES=n% 'number of times to sample each touch point
  DIM INTEGER XPT.VAR(2)
  DIM INTEGER XPT.IRQ=0, XPT.CS=1
  XPT.VAR(XPT.IRQ)=IRQ%:XPT.VAR(XPT.CS)=CS%
  IF XPT.NSAMPLES=0 THEN XPT.NSAMPLES=5
  DIM INTEGER XPT.rbuf(400) 'buffer area to support scrolling
  'Global variables to hold calibration values, can be assigned here if calibration already run
  DIM INTEGER XPT.XLEFT0=xl%, XPT.XRIGHT0=xr%, XPT.YTOP0=yt%, XPT.YBOTTOM0=yb%, XPT.XSAMPLE0=xs%, XPT.YSAMPLE0=ys% 
  ' global variables to hold active values
  DIM INTEGER XPT.XLEFT, XPT.XRIGHT, XPT.YTOP, XPT.YBOTTOM 
  ' arrays to hold raw sample values
  DIM INTEGER XPT.Xvalues(XPT.NSAMPLES-1),XPT.Yvalues(XPT.NSAMPLES-1)
  'parameters to hold XPT commands, these may be reversed by the rotate routine
  DIM INTEGER XPT.XSAMPLE=0, XPT.YSAMPLE 
'
  PIN(XPT.VAR(XPT.CS))=1
  SETPIN XPT.VAR(XPT.IRQ),DIN
  SETPIN XPT.VAR(XPT.CS),DOUT
  SPI OPEN 1000000,3,8 
  IF xl%=0 OR xr%=0 OR yt%=0 OR yb%=0 OR xs%=0 OR ys%=0 THEN XPT.CALIBRATE
END SUB
'
SUB XPT.Rotate(r AS INTEGER)
  S1963.ROTATE(r)'set the screen rotation
  IF S1963.Default_Rotation\2=r\2 THEN
    XPT.XSAMPLE=XPT.XSAMPLE0
    XPT.YSAMPLE=XPT.YSAMPLE0
    IF r=S1963.Default_Rotation THEN
      XPT.XLEFT=XPT.XLEFT0: XPT.XRIGHT=XPT.XRIGHT0 : XPT.YTOP=XPT.YTOP0 : XPT.YBOTTOM=XPT.YBOTTOM0
    ELSE
      XPT.XLEFT=XPT.XRIGHT0: XPT.XRIGHT=XPT.XLEFT0 : XPT.YTOP=XPT.YBOTTOM0 : XPT.YBOTTOM=XPT.YTOP0
    ENDIF
  ELSE
    XPT.XSAMPLE=XPT.YSAMPLE0
    XPT.YSAMPLE=XPT.XSAMPLE0
    IF ((r MOD 2) = (S1963.Default_Rotation MOD 2)) THEN
      IF S1963.Default_Rotation=2 THEN
         XPT.XLEFT=XPT.YBOTTOM0
         XPT.XRIGHT=XPT.YTOP0
         XPT.YTOP=XPT.XLEFT0
         XPT.YBOTTOM=XPT.XRIGHT0
       ELSE
         XPT.XLEFT=XPT.YTOP0
         XPT.XRIGHT=XPT.YBOTTOM0
         XPT.YTOP=XPT.XRIGHT0
         XPT.YBOTTOM=XPT.XLEFT0
       ENDIF
    ELSE
      IF S1963.Default_Rotation=0 THEN
         XPT.XLEFT=XPT.YBOTTOM0
         XPT.XRIGHT=XPT.YTOP0
         XPT.YTOP=XPT.XLEFT0
         XPT.YBOTTOM=XPT.XRIGHT0
       ELSE
         XPT.XLEFT=XPT.YTOP0
         XPT.XRIGHT=XPT.YBOTTOM0
         XPT.YTOP=XPT.XRIGHT0
         XPT.YBOTTOM=XPT.XLEFT0
       ENDIF
     ENDIF
   ENDIF
END SUB
'
SUB XPT.Calibrate 
  LOCAL INTEGER x,y,rsz=(S1963.MM.VRES+S1963.MM.HRES)\100, roff=rsz\2, csz=rsz\3
  PRINT ""
  XPT.XSAMPLE0=&H90: XPT.YSAMPLE0=&HD0
  S1963.ROTATE(S1963.Default_Rotation)
  XPT.XSAMPLE=XPT.XSAMPLE0
  XPT.YSAMPLE=XPT.YSAMPLE0
  IF (rsz MOD 2) THEN rsz=rsz+1
  IF csz=0 THEN csz=1
  SWITCHXY:
  S1963.CLS(0) 
  S1963.BOX(0,S1963.MM.VRES\2-roff,rsz,rsz,rgb(white),rgb(white))
  S1963.TEXT(S1963.MM.HRES\2-csz*4*8,S1963.MM.VRES\2-csz*4,"Touch box",0,csz,rgb(green),0)
  XPT.GetTouch(XPT.XLEFT0,y)
  IF XPT.XLEFT0>1000 AND XPT.XLEFT0<3000 THEN 'Switching orientation
    XPT.XSAMPLE0=&HD0:XPT.XSAMPLE=XPT.XSAMPLE0
    XPT.YSAMPLE0=&H90:XPT.YSAMPLE=XPT.YSAMPLE0
    GOTO SWITCHXY
  ENDIF
  PRINT "XPT.XSAMPLE0=",HEX$(XPT.XSAMPLE0)
  PRINT "XPT.YSAMPLE0=",HEX$(XPT.YSAMPLE0)
  PRINT "XPT.XLEFT0=",XPT.XLEFT0
  S1963.CLS(0) : PAUSE 1000
  S1963.BOX(S1963.MM.HRES-rsz,S1963.MM.VRES\2-roff,rsz,rsz,rgb(white),rgb(white))
  S1963.TEXT(S1963.MM.HRES\2-csz*4*8,S1963.MM.VRES\2-csz*4,"Touch box",0,csz,rgb(green),0)
  XPT.GetTouch(XPT.XRIGHT0,y)
  PRINT "XPT.XRIGHT0=",XPT.XRIGHT0
  S1963.CLS(0) : PAUSE 1000
  S1963.BOX(S1963.MM.HRES\2-roff,0,rsz,rsz,rgb(white),rgb(white))
  S1963.TEXT(S1963.MM.HRES\2-csz*4*8,S1963.MM.VRES\2-csz*4,"Touch box",0,csz,rgb(green),0)
  XPT.GetTouch(x,XPT.YTOP0)
  PRINT "XPT.YTOP0=",XPT.YTOP0
  S1963.CLS(0) : PAUSE 1000
  S1963.BOX(S1963.MM.HRES\2-roff,S1963.MM.VRES-rsz,rsz,rsz,rgb(white),rgb(white))
  S1963.TEXT(S1963.MM.HRES\2-csz*4*8,S1963.MM.VRES\2-csz*4,"Touch box",0,csz,rgb(green),0)
  XPT.GetTouch(x,XPT.YBOTTOM0)
  PRINT "XPT.YBOTTOM0=",XPT.YBOTTOM0
  S1963.CLS(0) : PAUSE 1000
  XPT.XLEFT=XPT.XLEFT0: XPT.XRIGHT=XPT.XRIGHT0 : XPT.YTOP=XPT.YTOP0 : XPT.YBOTTOM=XPT.YBOTTOM0 ' set the parameters for the default orientation
  'for a particular application these values can be saved and set as CONSTants
END SUB
    '
FUNCTION XPT.Screenpos(x AS INTEGER, y AS INTEGER, t%) AS INTEGER
    LOCAL INTEGER j
    IF XPT.GetXY(XPT.Xvalues(),XPT.Yvalues(),t%,XPT.NSAMPLES)<>0 THEN
      '      j=shellsortARM(XPT.Xvalues(),XPT.NSAMPLES) 'uncomment to use sort as required
      j=shellsort(XPT.Xvalues(),XPT.NSAMPLES) 'uncomment to use sort as required
      '      j=shellsortARM(XPT.Yvalues(),XPT.NSAMPLES) 'uncomment to use sort as required
      j=shellsort(XPT.Yvalues(),XPT.NSAMPLES) 'uncomment to use sort as required
      x=XPT.Xvalues(XPT.NSAMPLES\2) 'get the median value
      y=XPT.Yvalues(XPT.NSAMPLES\2) 'get the median value
      IF XPT.XLEFT>XPT.XRIGHT THEN 'x coordinates reversed
        IF x<XPT.XRIGHT THEN x=XPT.XRIGHT
        IF x>XPT.XLEFT THEN x=XPT.XLEFT
        x=S1963.MM.HRES-((x-XPT.XRIGHT)*S1963.MM.HRES/(XPT.XLEFT-XPT.XRIGHT))
      ELSE 'x coordinates normal
        IF x<XPT.XLEFT THEN x=XPT.XLEFT
        IF x>XPT.XRIGHT THEN x=XPT.XRIGHT
        x=(x-XPT.XLEFT)*S1963.MM.HRES/(XPT.XRIGHT-XPT.XLEFT)
      ENDIF
      IF XPT.YTOP>XPT.YBOTTOM THEN ' y coordinates reversed
        IF y>XPT.YTOP THEN y=XPT.YTOP
        IF y<XPT.YBOTTOM THEN y=XPT.YBOTTOM
        y=S1963.MM.VRES-((y-XPT.YBOTTOM)*S1963.MM.VRES/(XPT.YTOP-XPT.YBOTTOM))
      ELSE 'y coordinates normal
        IF y<XPT.YTOP THEN y=XPT.YTOP
        IF y>XPT.YBOTTOM THEN y=XPT.YBOTTOM
        y=(y-XPT.YTOP)*S1963.MM.VRES/(XPT.YBOTTOM-XPT.YTOP)
      ENDIF
      XPT.Screenpos=1
    ELSE
      XPT.Screenpos=0
    ENDIF
END FUNCTION
    
SUB XPT.GetTouch(x AS INTEGER, y AS INTEGER)
    LOCAL INTEGER j, n=51,XPT.Xvalues(n-1),XPT.Yvalues(n-1)
loophere:
    x=-500
    IF XPT.GetXY(XPT.Xvalues(),XPT.Yvalues(),1000,n)<>0 THEN
      j=shellsort(XPT.Xvalues(),n) 'change to use sort as required
      j=shellsort(XPT.Yvalues(),n) 'change to use sort as required
      x=XPT.Xvalues(n\2) 'get the median value
      y=XPT.Yvalues(n\2) 'get the median value
    ENDIF
    IF x=-500 THEN GOTO loophere
END SUB
    '
FUNCTION XPT.GetData(command AS INTEGER) AS INTEGER 'get data from the touch controller
    LOCAL INTEGER i,j,k
    PIN(XPT.VAR(XPT.CS))=0
    i=SPI(command)
    j=SPI(0)
    k=SPI(0)
    PIN(XPT.VAR(XPT.CS))=1
    XPT.GetData= (j*256+k)>>4
END FUNCTION
    '
FUNCTION XPT.GetXY(x() AS INTEGER, y() AS INTEGER, timeout AS INTEGER, samples AS INTEGER) AS INTEGER
    LOCAL INTEGER i
    TIMER=0
    XPT.GetXY=1
    FOR i=0 TO samples-1
      DO WHILE TIMER<timeout AND PIN(XPT.VAR(XPT.IRQ))=1:LOOP
      IF TIMER>=timeout THEN
        XPT.GetXY=0
      ELSE
        y(i)=XPT.GetData(XPT.YSAMPLE)
        x(i)=XPT.GetData(XPT.XSAMPLE)
      ENDIF
    NEXT i
END FUNCTION
'
CFUNCTION S1963
	000005E4
	'psend
	8ca200a0 8ca300a4 00431025 10400005 8ca70098 3c02bf88 ac446230 10000013 
	8ce20010 8ca60030 00863026 aca40030 000427c3 aca40034 24e20024 24e50074 
	24030001 00662024 50800005 24420008 8c440000 8c480004 ac880000 24420008 
	1445fff8 00031840 8ce20010 8ce30014 ac430000 8ce2000c 8ce30014 ac430000 
	03e00008 00000000 

	'wc
	27bdffe8 afbf0014 8ca20098 8c430004 8c420008 ac620000 0411FFD7 00000000 
	8fbf0014 03e00008 27bd0018 

	'wd
	27bdffe8 afbf0014 8ca20098 8c430000 8c420008 ac620000 0411FFCC 00000000 
	8fbf0014 03e00008 27bd0018 

	'initdisplay
	27bdffc8 afbf0034 afb70030 afb6002c afb50028 afb40024 afb30020 afb2001c 
	afb10018 afb00014 0080b021 3c109d00 8e02003c 0040f809 24040080 0040b821 
	8e020024 8ec40038 0040f809 24050006 aee20000 8e020024 8ec40038 0040f809 
	24050005 aee20004 8e020028 0040f809 8ec40038 24110001 00511004 aee20008 
	8e020024 8ec40040 0040f809 24050006 aee2000c 8e020024 8ec40040 0040f809 
	24050005 aee20010 8e020028 0040f809 8ec40040 00511004 aee20014 02c09821 
	26f00024 26f400a4 02c08821 3c129d00 24150001 8e420024 8e240048 0040f809 
	24050007 ae020000 8e420028 0040f809 8e240048 00551004 ae020004 26100008 
	1614fff4 26310008 24110007 3c109d00 24120012 8e020010 8e640038 24050008 
	0040f809 00003021 8e02001c 8e640038 0040f809 24050005 26310001 1632fff5 
	26730008 3c109d00 8e02001c 8ec40038 0040f809 24050006 8e02001c 8ec40040 
	0040f809 24050006 8ec40090 8ec20094 00821025 5040001a 8ec200a0 8e020024 
	0040f809 24050006 aee20018 8e020024 8ec40090 0040f809 24050005 aee2001c 
	8e020028 0040f809 8ec40090 24030001 00431004 aee20020 8e020010 8ec40090 
	24050008 0040f809 00003021 8e02001c 8ec40090 0040f809 24050006 8ec200a0 
	8ec300a4 00431025 1040000c 02e01021 3c109d00 8e020010 24040005 24050008 
	0040f809 00003021 8e02001c 24040005 0040f809 24050005 02e01021 00001821 
	8fbf0034 8fb70030 8fb6002c 8fb50028 8fb40024 8fb30020 8fb2001c 8fb10018 
	8fb00014 03e00008 27bd0038 

	'defineregion
	27bdffd0 afbf002c afb50028 afb40024 afb30020 afb2001c afb10018 afb00014 
	8fb10044 2482ffff 00461021 24a3ffff 00671821 8e260010 8e270000 8e280008 
	14c00008 8e300098 00a09021 00609821 0002a027 0288a021 0004a827 10000012 
	02a8a821 24090001 14c90008 24090002 00031827 00679021 00059827 02679821 
	0080a021 10000012 0040a821 14c90007 24090003 00809021 00409821 00a0a021 
	1000000b 0060a821 24090003 14c90008 00021027 00479021 00049827 02679821 
	00031827 0068a021 0005a827 02a8a821 8e020004 8e030008 ac430000 2404002a 
	02202821 0411FEFB 00000000 8e020000 8e030008 ac430000 00122202 02202821 
	0411FEF4 00000000 02402021 02202821 0411FEF0 00000000 00132202 02202821 
	0411FEEC 00000000 02602021 02202821 0411FEE8 00000000 8e020004 8e030008 
	ac430000 2404002b 02202821 0411FEE1 00000000 8e020000 8e030008 ac430000 
	00142202 02202821 0411FEDA 00000000 02802021 02202821 0411FED6 00000000 
	00152202 02202821 0411FED2 00000000 02a02021 02202821 0411FECE 00000000 
	8e020004 8e030008 ac430000 8fa20040 10400007 2404002e 2404002c 02202821 
	0411FEC4 00000000 10000005 8e020000 02202821 0411FEBF 00000000 8e020000 
	8e030008 ac430000 8fbf002c 8fb50028 8fb40024 8fb30020 8fb2001c 8fb10018 
	8fb00014 03e00008 27bd0030 

	'localfastfillrect
	27bdffc8 afbf0034 afb50030 afb4002c afb30028 afb20024 afb10020 afb0001c 
	0080a021 8fa20048 8fb1004c 8e300098 7047a802 24030001 afa30010 afb10014 
	00a02021 00c02821 00e03021 00403821 0411FF68 00000000 7e821340 001493c3 
	325201f8 00529025 7e932880 0014a103 329401c0 02749825 8e020000 8e030008 
	ac430000 12530006 02402021 26b0ffff 16a0001f 2414ffff 10000028 24020001 
	02202821 0411FE88 00000000 02402021 02202821 0411FE84 00000000 26b5ffff 
	12a0001d 26b5ffff 2404ffff 8e020010 8e030014 ac430000 8e02000c 8e030014 
	ac430000 8e020010 8e030014 ac430000 8e02000c 8e030014 ac430000 26b5ffff 
	56a4fff3 8e020010 1000000c 24020001 02202821 0411FE6C 00000000 02602021 
	02202821 0411FE68 00000000 2610ffff 1614fff7 02402021 24020001 00001821 
	8fbf0034 8fb50030 8fb4002c 8fb30028 8fb20024 8fb10020 8fb0001c 03e00008 
	27bd0038 

	'DBitmap
	27bdff80 afbf007c afbe0078 afb70074 afb60070 afb5006c afb40068 afb30064 
	afb20060 afb1005c afb00058 afa40080 00a08021 afa60088 afa7008c 8fb20090 
	8fa30094 8fbe00a0 8fc40010 2c840002 10800005 8fa20098 8fd30008 8fc40000 
	10000004 afa40040 8fd30000 8fc40008 afa40040 7c651340 000323c3 308401f8 
	00a42025 afa4002c 7c642880 00031903 306301c0 00831825 afa30030 7c441340 
	00021bc3 306301f8 00831825 afa30034 7c432880 00021143 304201c0 00621025 
	afa20038 24020001 afa20010 afbe0014 8fa40080 02002821 8fa20088 72423002 
	8fa2008c 72423802 0411FEE9 00000000 3c029d00 8c42001c 8fc40038 0040f809 
	24050006 8fa3008c 18600072 8fbf007c afb20024 afb00044 8fa40088 afa40050 
	00041023 afa20054 00640018 00001012 afa2004c afa00020 10000062 afa00048 
	0600001e 0213102b 5040001d 26310001 06a2001b 26310001 52e00019 26310001 
	8fa3001c 90620000 8fa40018 00441024 1040000b 8fa40034 8fa4002c 03c02821 
	0411FDF8 00000000 8fa40030 03c02821 0411FDF4 00000000 10000009 26310001 
	03c02821 0411FDEF 00000000 8fa40038 03c02821 0411FDEB 00000000 26310001 
	1632ffdf 26100001 26940001 8fa20024 8fa30088 12830017 02c2b021 8fa40020 
	02841021 24430007 28440000 0064100b 000210c3 8fa3009c 00621021 afa2001c 
	8fa40028 00941823 000317c3 00021742 00621821 30630007 00621023 24030001 
	00431004 afa20018 02c08021 1000ffc4 00008821 8fa4003c 24840001 afa4003c 
	14920007 26b50001 1000000c 8fa20048 afa0003c 8fa2004c 2442ffff afa20028 
	8fa30088 1860fff3 8fb60080 0000a021 8fa40040 1000ffd9 02a4b82b 24420001 
	afa20048 8fa30044 8fa40024 00641821 afa30044 8fa20020 8fa30050 00431021 
	afa20020 8fa4004c 8fa20054 00822021 afa4004c 8fa30048 8fa4008c 10640005 
	8fbf007c 1e40ffe2 8fb50044 1000ffeb 8fa20048 8fbe0078 8fb70074 8fb60070 
	8fb5006c 8fb40068 8fb30064 8fb20060 8fb1005c 8fb00058 03e00008 27bd0080 

	'swap
	8c820000 8ca30000 ac830000 03e00008 aca20000 

	'drawline
	27bdffc0 afbf003c afb70038 afb60034 afb50030 afb4002c afb30028 afb20024 
	afb10020 afb0001c 0080a021 afa50044 afa60048 afa7004c 8fa20050 14c20016 
	8fb50054 00e5102a 10400006 8fa20044 27a40044 27a5004c 0411FFE4 00000000 
	8fa20044 8fa7004c 24e70001 24030001 afa30010 afb50014 02802021 00402821 
	8fa60048 00e23823 0411FEBF 00000000 10000049 8fbf003c 8fb60044 8fa3004c 
	16c30015 00769823 0046102a 10400006 8fa60048 27a40048 27a50050 0411FFCB 
	00000000 8fa60048 8fa20050 24420001 00461023 afa20010 afb50014 02802021 
	8fa50044 24070001 0411FEA7 00000000 10000031 8fbf003c 001327c3 00939826 
	02649823 02c3b02a 24040001 2405ffff 00b6200a 0080b021 00469023 00121fc3 
	00729026 00729023 00c2b82a 24030001 2404ffff 0097180a 0060b821 02538021 
	24110001 afb10010 afb50014 02802021 8fa50044 8fa60048 02203821 0411FE8A 
	00000000 00101040 0052182a 54600008 0262102a 8fa30044 8fa4004c 1064000d 
	02128021 02c31821 afa30044 0262102a 5440ffed afb10010 8fa20048 8fa30050 
	10430004 02138021 02e21021 1000ffe5 afa20048 8fbf003c 8fb70038 8fb60034 
	8fb50030 8fb4002c 8fb30028 8fb20024 8fb10020 8fb0001c 03e00008 27bd0040 

	'drawchar
	27bdffd0 afbf002c 8faa0040 8fa90044 8fa20048 8fab004c 7c042420 3c039d00 
	8c630054 00052880 00a31821 8c650000 90a80001 90ac0002 008c682a 15a00019 
	90a30000 90ad0003 01ac6821 008d682a 51a00015 71024002 afa20010 afa60014 
	afa70018 71031002 000210c3 008c6023 704c2002 24840004 00a42821 afa5001c 
	afab0020 01402021 01202821 00603021 01003821 0411FE9D 00000000 1000000b 
	8fbf002c 71024002 afa80010 afab0014 00e02021 01402821 01203021 70623802 
	0411FE39 00000000 8fbf002c 03e00008 27bd0030 

	'testchar
	8fa30010 8c680010 8c690000 70e61002 00442021 2502fffe 2c420002 10400009 
	8c630008 04800011 00001021 04a0000f 0124302a 14c0000d 0065302a 14c0000b 
	00000000 2d080002 11000008 24020001 04800006 00001021 04a00004 0064182a 
	14600002 0125282a 38a20001 03e00008 00000000 

	'pstring
	27bdffb8 afbf0044 afbe0040 afb7003c afb60038 afb50034 afb40030 afb3002c 
	afb20028 afb10024 afb00020 0080b821 00a08021 00c09821 00e0a021 8fb50064 
	3c029d00 8c420054 00041880 00621021 8c420000 90560000 92a20000 1840001d 
	8fb20060 72c7f002 24110001 afb20010 02002021 02602821 02803021 02c03821 
	0411FFC2 00000000 10400019 02b11021 80440000 afb00010 afb30014 afb40018 
	afb2001c 02e02821 8fa60058 8fa7005c 0411FF81 00000000 00404021 00604821 
	26310001 92a20000 0051102a 1040ffe7 021e8021 ae500020 001087c3 ae500024 
	ae530028 00139fc3 10000003 ae53002c 00004021 00004821 01001021 01201821 
	8fbf0044 8fbe0040 8fb7003c 8fb60038 8fb50034 8fb40030 8fb3002c 8fb20028 
	8fb10024 8fb00020 03e00008 27bd0048 

	'dcirch
	27bdffb0 afbf004c afbe0048 afb70044 afb60040 afb5003c afb40038 afb30034 
	afb20030 afb1002c afb00028 0080b021 00a0a821 00c0a021 00e08821 8fa30060 
	30620010 10400020 8fb70064 24100001 afb00010 afb70014 00e63021 24070001 
	0411FDB3 00000000 afb00010 afb70014 02c02021 02a02821 02913023 24070001 
	0411FDAB 00000000 afb00010 afb70014 02c02021 02352821 02803021 24070001 
	0411FDA3 00000000 afb00010 afb70014 02c02021 02b12823 02803021 24070001 
	0411FD9B 00000000 1a200069 24130001 02719823 0011f023 001ef040 00008021 
	8fa20060 30420004 afa2001c 24120001 8fa30060 30630002 afa30020 8fa20060 
	30420008 afa20024 8fa30060 30630001 afa30018 06600004 8fa2001c 2631ffff 
	27de0002 027e9821 10400011 26100001 afb20010 afb70014 02c02021 02152821 
	02343021 02403821 0411FD79 00000000 afb20010 afb70014 02c02021 02352821 
	02143021 02403821 0411FD71 00000000 8fa30020 10600012 8fa20024 afb20010 
	afb70014 02c02021 02152821 02913023 02403821 0411FD66 00000000 afb20010 
	afb70014 02c02021 02352821 02903023 02403821 0411FD5E 00000000 8fa20024 
	10400012 8fa30018 afb20010 afb70014 02c02021 02b12823 02143021 02403821 
	0411FD53 00000000 afb20010 afb70014 02c02021 02b02823 02343021 02403821 
	0411FD4B 00000000 8fa30018 10600012 0211102a afb20010 afb70014 02c02021 
	02b12823 02903023 02403821 0411FD40 00000000 afb20010 afb70014 02c02021 
	02b02823 02913023 02403821 0411FD38 00000000 0211102a 10400006 8fbf004c 
	00101040 24420001 1000ffaa 02629821 8fbf004c 8fbe0048 8fb70044 8fb60040 
	8fb5003c 8fb40038 8fb30034 8fb20030 8fb1002c 8fb00028 03e00008 27bd0050 

	'fcirch
	27bdffb0 afbf004c afbe0048 afb70044 afb60040 afb5003c afb40038 afb30034 
	afb20030 afb1002c afb00028 0080b821 00a0b021 8fb50068 18c00048 8fbe0070 
	24100001 02068023 0006a023 0014a040 00c08821 24130003 00009021 30e20001 
	afa20018 8fa30060 24630001 afa30020 30e70002 afa7001c 06000004 8fa20018 
	2631ffff 26940002 02148021 10400016 26520001 00111040 8fa30020 00621021 
	afa20010 afbe0014 02a02021 02572821 02d13023 24070001 0411FCF5 00000000 
	8fa30060 02631021 afa20010 afbe0014 02a02021 02372821 02d23023 24070001 
	0411FCEB 00000000 8fa2001c 10400017 0251102a 00111040 8fa30020 00621021 
	afa20010 afbe0014 02a02021 02f22823 02d13023 24070001 0411FCDD 00000000 
	8fa30060 02631021 afa20010 afbe0014 02a02021 02f12823 02d23023 24070001 
	0411FCD3 00000000 0251102a 10400003 02138021 1000ffc8 26730002 8fbf004c 
	8fbe0048 8fb70044 8fb60040 8fb5003c 8fb40038 8fb30034 8fb20030 8fb1002c 
	8fb00028 03e00008 27bd0050 

	'ftri
	27bdffc8 afbf0034 afb40030 afb3002c afb20028 afb10024 afb00020 afa40038 
	afa5003c afa60040 afa70044 8fb20050 00e5282a 10a00009 8fb10054 27a4003c 
	27a50044 0411FDC8 00000000 27a40038 27a50040 0411FDC4 00000000 8fa2004c 
	8fa30044 0043102a 1040000a 8fa20044 27a4004c 27a50044 0411FDBB 00000000 
	27a40048 27a50040 0411FDB7 00000000 8fa20044 8fa3003c 0043102a 1040000a 
	8fb0003c 27a4003c 27a50044 0411FDAE 00000000 27a40038 27a50040 0411FDAA 
	00000000 8fb0003c 8fa2004c 16020023 8fb30044 8fa20038 afa20018 afa2001c 
	8fa30040 0062202a 10800003 0043102a 10000003 afa30018 54400001 afa3001c 
	8fa20048 8fa30018 0043182a 10600003 8fa3001c 10000004 afa20018 0062182a 
	54600001 afa2001c 8fa50018 8fa2001c 24420001 00451023 afa20010 afb10014 
	02402021 8fa6003c 24070001 0411FC6D 00000000 10000068 8fbf0034 00531026 
	0002102b 02629823 0270102a 14400030 8fa2004c 24140001 8fa20038 8fa3003c 
	02032823 8fa60040 00c23023 70a63002 8fa40044 00832023 00c4001a 008001f4 
	00002012 00822021 afa40018 8fa60048 00c23023 70a62802 8fa6004c 00c31823 
	00a3001a 006001f4 00001812 00621021 afa2001c 0044102a 10400006 8fa20018 
	27a40018 27a5001c 0411FD5F 00000000 8fa20018 8fa7001c 24e70001 afb40010 
	afb10014 02402021 00402821 02003021 00e23823 0411FC3B 00000000 26100001 
	0270102a 1040ffd5 8fa20038 8fa2004c 0050182a 1460002f 24130001 8fa60040 
	8fa40038 8fa50048 8fa80044 02083823 00a61823 70e33802 00484023 00e8001a 
	010001f4 00001812 00661821 afa30018 8fa7003c 02073023 00a42823 70c53002 
	00471023 00c2001a 004001f4 00001012 00442021 afa4001c 0083202a 10800006 
	8fa20018 27a40018 27a5001c 0411FD2E 00000000 8fa20018 8fa7001c 24e70001 
	afb30010 afb10014 02402021 00402821 02003021 00e23823 0411FC0A 00000000 
	26100001 8fa2004c 0050182a 1060ffd4 8fa60040 8fbf0034 8fb40030 8fb3002c 
	8fb20028 8fb10024 8fb00020 03e00008 27bd0038 

	'scroll
	27bdffb0 afbf004c afbe0048 afb70044 afb60040 afb5003c afb40038 afb30034 
	afb20030 afb1002c afb00028 00808821 afa50054 afa7005c 8fb40068 24b5ffff 
	02a7a821 2482ffff 00461021 afa2001c 3c109d00 8e020028 0040f809 8e840090 
	24120001 00529004 8e020024 8e840090 0040f809 24050005 0040b821 8e020024 
	8e840090 0040f809 24050006 0040b021 26310001 afb10018 8fa3001c 0071102a 
	1440004b 3c10bf88 8fa40054 0095202a afa40020 3c1ebf81 afa00010 afb40014 
	8fa40018 8fa50054 24060001 8fa7005c 0411FB44 00000000 240201ff ae026210 
	3c039d00 8c62001c 8e840038 0040f809 24050006 8fa40020 10800041 8fa30054 
	8fb10064 02201021 aef20000 8e046210 8fc4f220 aed20000 8e046220 ac440000 
	aef20000 8e046210 8fc4f220 aed20000 8e046220 ac440004 24630001 1475fff2 
	24420008 ae006210 24020001 afa20010 afb40014 8fa20018 2444ffff 8fa50054 
	24060001 8fa7005c 0411FB1E 00000000 3c039d00 8c62001c 8e840038 0040f809 
	24050006 8fb30054 8e240000 02802821 0411FA49 00000000 8e240004 02802821 
	0411FA45 00000000 26730001 1675fff6 26310008 8fa40018 24840001 afa40018 
	8fa3001c 0064102a 5040ffbc afa00010 8fbf004c 8fbe0048 8fb70044 8fb60040 
	8fb5003c 8fb40038 8fb30034 8fb20030 8fb1002c 8fb00028 03e00008 27bd0050 
	ae006210 24020001 afa20010 afb40014 8fa20018 2444ffff 8fa50054 24060001 
	8fa7005c 0411FAEF 00000000 3c039d00 8c62001c 8e840038 0040f809 24050006 
	1000ffdd 8fa40018 

	'main
	27bdffd0 afbf002c afb00028 00808021 8fa80040 8faa0044 8fa90048 8fab004c 
	8c820000 8c830004 00432025 1480000d 8fac0050 8ca40000 8cc50000 8ce60000 
	24020001 afa20010 afa80014 24070001 0411FB55 00000000 00402021 100000b2 
	00602821 24040001 1444000f 24040002 1460000d 00000000 8ca40000 8cc50000 
	8ce60000 8d070000 8d420000 afa20010 afa90014 0411FB44 00000000 00402021 
	100000a1 00602821 14440013 24040003 14600011 00000000 8ca40000 8cc50000 
	8ce60000 8d070000 8d420000 afa20010 8d220000 afa20014 8d620000 afa20018 
	afac001c 0411FE6F 00000000 8e040000 1000008d 8e050004 1444000f 24040004 
	1460000d 00000000 8ca40000 8cc50000 8ce60000 8d070000 8d420000 afa20010 
	afa90014 0411FC3E 00000000 8e040000 1000007d 8e050004 1444000a 24040005 
	14600008 00000000 8ca40000 00c02821 0411F9E9 00000000 8e040000 10000072 
	8e050004 1444000a 24040007 14600008 00000000 8ca40000 00c02821 0411F9E9 
	00000000 8e040000 10000067 8e050004 14440012 24040008 14600010 00000000 
	8ca40000 8cc50000 8ce60000 8d070000 8d420000 afa20010 8d220000 afa20014 
	afab0018 afac001c 0411FCDF 00000000 00402021 10000054 00602821 14440013 
	24040009 14600011 00000000 8ca40000 8cc50000 8ce60000 8d070000 8d420000 
	afa20010 8d220000 8d230004 afa20018 afa3001c afab0020 0411FDBF 00000000 
	8e040000 10000040 8e050004 1444000f 2404000a 1460000d 00000000 8ca40000 
	8cc50000 8ce60000 8d070000 8d420000 afa20010 afa90014 0411FD07 00000000 
	8e040000 10000030 8e050004 14440015 2404000b 14600013 00000000 8ca40000 
	8cc50000 8ce60000 8d070000 8d420000 afa20010 8d220000 afa20014 8d620000 
	afa20018 afac001c 8fa20054 afa20020 0411FB16 00000000 00002021 1000001a 
	00002821 14440010 2404000c 1460000e 00000000 8ca40000 8cc50000 8ce60000 
	8d070000 8d420000 afa20010 afa90014 afab0018 0411FEB0 00000000 00402021 
	10000009 00602821 1444000a 8fbf002c 14600008 00a02021 0411F98D 00000000 
	00402021 00602821 00801021 00a01821 8fbf002c 8fb00028 03e00008 27bd0030 
END CFUNCTION   'MIPS32 M4K
'
CFUNCTION shellsort
  00000000
  8ca20000 27bdfff8 afb10004 afb00000 00401821 00032fc2 00a31821 00031843
  5060003d 00001021 0062282a 10a0fffa 00032fc2 00036823 000370c0 000d60c0
  008e7021 0080c021 008c8021 00607821 0000c821 01e3282a 01c03021 8dd10000
  14a0001e 8dcb0004 8f070004 03202821 0167402a 11000034 8f090000 01e04021
  000840c0 00884021 00a3502a 02003021 ad090000 1540000f ad070004 8cc70004
  8cc90000 0167402a 00ad5021 1100001f 00cc3021 00a04021 01402821 000840c0
  00884021 00a3502a ad090000 1140fff3 ad070004 000528c0 00853021 25ef0001
  01e2282a acd10000 accb0004 27390001 25ce0008 27180008 14a0ffd6 26100008
  00032fc2 00a31821 00031843 1460ffc7 0062282a 00001021 00001821 8fb10004
  8fb00000 03e00008 27bd0008 14ebffe9 0229402b 5100ffe8 000528c0 00a04021
  1000ffde 01402821 54ebffe5 25ef0001 0229402b 5500ffca 01e04021 1000ffe0
  25ef0001
END CFUNCTION

'----------------------------------------------------------------------------
	'Cut here and throw away below when using the Library in your own program
testsuite:
	
	PRINT "start Init"
	
	'Tell the library which pins we're using for the display
	'Connect CS to GND
	'SUB S1963.PinDefs(D0,D1,D2,D3,D4,D5,D6,D7,D8,D9,RS,RD,WR,RST)
	S1963.PinDefs(25,26,27,36,37,38,2,3,4,5,12,13,23,32)
	S1963.init(4.3) 'initialise the S1963 display
	
	'CS  = pin 42  ' chip select pin for XPT touch controller, any valid pin
	'IRQ = pin 43  ' interrupt pin for XPT touch controller, any valid pin
	' Connect T_DO to SPIIN (pin 14 on 28-pin, pin 41 on 44-pin)
	' Connect T_CLK to SPICLK (pin 25 on 28-pin,pin 14 on 44-pin)
	' Connect T_DIN to SPIOUT (pin 3 on 28-pin, pin 20 on 44-pin)

	'SUB XPT.init(IRQ,CSxl%, xr%, yt%, yb%, xs%, ys%, n%)
	'initialise the touch system, just supply the IRQ & CS pin numbers 
	'no paramaters so calibration will be instigated
	XPT.Init(43,42) 
	
	PRINT "Start main"

	DO
	   S1963TestSuite
	LOOP

END


SUB S1963TestSuite 'program to test and demonstrate all functions
  LOCAL INTEGER k,x,y 'main program variables
  LOCAL INTEGER i, j, x0, y0, x1, y1, sz, bitmap(1)=(&HFFC1A191898583FF,0)
  LOCAL INTEGER rp=130, bp=100, yp=160, wp=190, cp=70, xp=40, zp=220
  LOCAL FLOAT tn
  CONST scrollxstart=100, scrollxwidth=280,newpoint=scrollxstart+scrollxwidth-1
  CONST scrollystart=20,scrollyheight=232, scrolldirection=0
  S1963.ROTATE(LANDSCAPE)
  S1963.CLS()
  S1963.BOX(0, 0,S1963.MM.HRES,S1963.MM.VRES,rgb(red))
  FOR i=10 TO 0 STEP -1
    S1963.TEXT(S1963.MM.HRES/2-48, 70, STR$(i,2) ,1 ,2, rgb(green))
    PAUSE 1000
  NEXT i
  S1963.TEXT(S1963.MM.HRES/2-48, 70, "  " ,1 ,2, rgb(green))
  FOR i=0 TO 3
     S1963.ROTATE(i)
     S1963.BITMAP(10,10,bitmap(),8,8,3,rgb(blue))
  NEXT i  
  S1963.ROTATE(LANDSCAPE)
  S1963.TEXT(S1963.MM.HRES/2-112, 60, "Welcome to the", 0,2, rgb(green))
  S1963.TEXT(S1963.MM.HRES/2-168, 100,  "Micromite MKII 4.7b8", 0,2, rgb(red))
  S1963.TEXT(S1963.MM.HRES/2-108, 150, "in colour", 0,3, rgb(yellow))
  S1963.TEXT(S1963.MM.HRES/2-176, 220,  "9-bit parallel S1963", 0,2, rgb(cyan))
  PAUSE 4000
' demo fast line draw
  S1963.CLS()
  S1963.ROTATE(RLANDSCAPE)
  x0=0:y0=0:y1=S1963.MM.VRES-1
  FOR i=0 TO 44 STEP 2
    tn=TAN(RAD(i))
    x1=S1963.MM.HRES*tn
    S1963.LINE(x0, y0, x1, y1, rgb(red))
  NEXT i
  x1=S1963.MM.HRES-1
  FOR i=0 TO 44 STEP 2
    tn=TAN(RAD(i))
    y1=S1963.MM.VRES*tn
    S1963.LINE(x0, y0, x1, y1, rgb(red))
  NEXT i
  S1963.BOX(0, 0, S1963.MM.HRES, S1963.MM.VRES, rgb(blue))
  PAUSE 2000

  S1963.CLS()
'demo fast text write
  S1963.ROTATE(LANDSCAPE)

  FOR i=0 TO S1963.MM.VRES-13 STEP 13
    S1963.TEXT(0, i, "Hello world:Bye world:Hello world:Bye world:Hello world:Bye world:Hello world:Bye world", 0,1, rgb(white))
  NEXT i
  PAUSE 2000
  S1963.ROTATE(RLANDSCAPE)
  S1963.CLS()
  FOR i=0 TO S1963.MM.VRES-26 STEP 26
    S1963.TEXT(0, i, "Micromite MK2:Invert:Micromite MK2:Invert", 0,2, rgb(red))
  NEXT i
  PAUSE 2000
  S1963.ROTATE(RPORTRAIT)
  S1963.CLS()
  FOR i=0 TO S1963.MM.VRES-39 STEP 39
    S1963.TEXT(0,i, "Rotate right:Rotate right:Rotate right", 0,3, rgb(yellow))
  NEXT i
  PAUSE 2000
  S1963.ROTATE(PORTRAIT)
  S1963.CLS()
  FOR i=0 TO S1963.MM.VRES-52 STEP 52
    S1963.TEXT(0,i, "Rotate Left:Rotate Left:Rotate Left", 0,4, rgb(blue))
  NEXT i
  PAUSE 2000
  S1963.CLS()
  ' test drawing rectangles
  FOR i=0 TO 3
     S1963.ROTATE(i)
     S1963.BOX(5, 5, 80, 90, rgb(blue), rgb(blue))
     S1963.BOX(2, 2, 86, 96, rgb(red))
  NEXT i
  PAUSE 2000
' test drawing rounded rectangles
  S1963.CLS()
  FOR i=0 TO 3
     S1963.ROTATE(i)
     S1963.RBOX(5, 5, 80, 45, 10, rgb(blue), rgb(blue))
     S1963.RBOX(2, 2, 86, 51, 12, rgb(red))
  NEXT i
  PAUSE 2000
  S1963.CLS()
' test drawing circles
  FOR i=0 TO 3
     S1963.ROTATE(i)
     S1963.CIRCLE(40, 40, 35, rgb(blue), rgb(blue))
     S1963.CIRCLE(40, 40, 39, rgb(red))
  NEXT i
  PAUSE 2000
' test drawing triangles and line drawing
  S1963.CLS()
  FOR i=0 TO 3
     S1963.ROTATE(i)
     S1963.TRIANGLE(4, 4, 140, 6, 22, 124, rgb(blue), rgb(blue))
     S1963.TRIANGLE(1, 2, 144, 4, 22, 127, rgb(red))
  NEXT i
  PAUSE 2000
  S1963.ROTATE(RLANDSCAPE)
  S1963.CLS()
  S1963.BOX(scrollxstart-1,scrollystart-1,scrollxwidth+2,scrollyheight+2,rgb(red))
  S1963.TEXT(scrollxstart+8,0,"Micromite MKII 4.7b8 with S1963",0,1,rgb(white))
  S1963.TEXT(newpoint+10,scrollystart-13+(scrollyheight\2),"Fixed",0,2,rgb(green))
  S1963.TEXT(newpoint+10,scrollystart+13+(scrollyheight\2),"Area",0,2,rgb(green))
  S1963.TEXT(10,scrollystart-13+(scrollyheight\2),"Fixed",0,2,rgb(green))
  S1963.TEXT(10,scrollystart+13+(scrollyheight\2),"Area",0,2,rgb(green))
  S1963.TEXT(scrollxstart+12,scrollystart+scrollyheight+5,"Demonstration of scrolling data",0,1,rgb(white))
 TIMER=0
 DO
   S1963.SCROLL(scrollxstart,scrollystart,scrollxwidth,scrollyheight)
   S1963.PIXEL(newpoint,xp) ' clear the old data after the scroll
   S1963.PIXEL(newpoint,rp)
   S1963.PIXEL(newpoint,yp)
   S1963.PIXEL(newpoint,bp)
   S1963.PIXEL(newpoint,wp)
   S1963.PIXEL(newpoint,cp)
   S1963.PIXEL(newpoint,zp)
   rp=rp+(RND()*4)-2 'get the new data points
   bp=bp+(RND()*4)-2
   yp=yp+(RND()*4)-2
   xp=xp+(RND()*2)-1
   zp=zp+(RND()*4)-2
   wp=wp+(RND()*4)-2
   cp=cp+(RND()*4)-2
   S1963.PIXEL(newpoint,xp,rgb(yellow)) 'write them to the space left
   S1963.PIXEL(newpoint,rp,rgb(blue))
   S1963.PIXEL(newpoint,yp,rgb(red))
   S1963.PIXEL(newpoint,bp,rgb(green))
   S1963.PIXEL(newpoint,wp,rgb(magenta))
   S1963.PIXEL(newpoint,cp,rgb(cyan))
   S1963.PIXEL(newpoint,zp,rgb(white))
LOOP UNTIL TIMER>40000
  FOR j=0 TO 3
     XPT.Rotate(j)
     S1963.CLS(0)
     S1963.TEXT(0,0,"Top left",0,2,rgb(green))
     k=0
main:
     IF XPT.Screenpos(x,y,1000) THEN
       IF x>3 AND x<S1963.MM.HRES-3 AND y>3 AND y<S1963.MM.VRES-3 THEN S1963.CIRCLE(x,y,3,rgb(white),rgb(white))
     ELSE 
        k=k+1
     ENDIF
     IF k<10 GOTO main
  NEXT j
END SUB
